"""
These are the options that we allow for the
video generation. If the option is not here,
it is not available.

TODO: Check 'yta-api' project to see a 
broader implementation.
"""
from yta_general_utils.programming.enum import YTAEnum as Enum


class VoiceNarrationEngine(Enum):
    DEFAULT = 'default'
    """
    When this option is provided, the system will
    choose one of the available enum elements.
    """
    GOOGLE = 'google'

class VoiceNarrationNarratorName(Enum):
    DEFAULT = 'default'
    """
    When this option is provided, the system will
    choose one of the available enum elements.
    """

class VoiceNarrationSpeed(Enum):
    DEFAULT = 'default'
    """
    When this option is provided, the system will
    choose one of the available enum elements.
    """
    NORMAL = 'normal'

class VoiceNarrationEmotion(Enum):
    DEFAULT = 'default'
    """
    When this option is provided, the system will
    choose one of the available enum elements.
    """
    NORMAL = 'normal'

class VoiceNarrationPitch(Enum):
    DEFAULT = 'default'
    """
    When this option is provided, the system will
    choose one of the available enum elements.
    """
    NORMAL = 'normal'


"""
This is a complex parameter which includes
different attributes to make possible the
music construction.

This parameter can be None if no music is
requested, or can have different values
according to the expected way of building
it.
"""
from yta_core.builder.music.enums import MusicEngine
from dataclasses import dataclass
from typing import Union

import json


@dataclass
class MusicJson:
    """
    @dataclass
    The information the Music parameter must hold.
    This information is about a music that is going
    to be added in a video.
    """

    filename: str
    url: str
    engine: Union[MusicEngine, None]
    keywords: str
    filename_processed: str
    """
    The filename that has been built and/or processed
    and is able to be used for the Segment or 
    Enhancement.
    """

    def __init__(self):
        """
        Initializes the instance with empty string
        values for all the attributes so we can
        fill them later and individually when
        processing.
        """
        self.filename = ''
        self.url = ''
        self.engine = None
        self.keywords = ''
        self.filename_processed = ''

    @property
    def as_dict(self):
        """
        Get the instance as a dictionary.
        """
        return {
            'filename': self.filename,
            'url': self.url,
            'engine': (
                self.engine.value
                if self.engine is not None else
                None
            ),
            'keywords': self.keywords,
            'filename_processed': self.filename_processed
        }

    @property
    def as_json(self):
        """
        Get the instance as a json, transformed
        from the dictionary version.
        """
        return json.dumps(self.as_dict)


from yta_general_utils.programming.enum import YTAEnum as Enum


class MusicEngine(Enum):
    DEFAULT = 'default'
    """
    When this option is provided, the system will
    choose one of the available enum elements.
    """
    YOUTUBE = 'youtube'
    """
    The music will be generated by downloading it
    from our specific Youtube channel.
    """


from yta_core.builder.voice_narration.enums import VoiceNarrationEmotion,VoiceNarrationEngine, VoiceNarrationSpeed, VoiceNarrationNarratorName
from yta_core.enums.field_v2 import _Field
from yta_core.validation import VoiceNarrationField, VoiceNarrationEmotionField, VoiceNarrationEngineField, VoiceNarrationLanguageField, VoiceNarrationFilenameField, VoiceNarrationNarratorNameField, VoiceNarrationSpeedField, VoiceNarrationTextField
from yta_shortcodes.parser import ShortcodeParser
from yta_audio.voice.generation.narrator import GoogleVoiceNarrator
from yta_general_utils.programming.validator.parameter import ParameterValidator
from typing import Union


class VoiceNarrationConcept:
    """
    Class to simplify the way we handle the 
    'voice_narration' for a component.

    This class must be used when the component json
    structure has been validated and it is ready for
    the building process.

    TODO: Check 'validation.py' for the validation
    process.
    """

    @staticmethod
    def do_should_be_handled(
        element: dict
    ) -> bool:
        """
        Check if the 'voice_narration' should be handled
        or not according to the parameters set in the
        given 'element'.

        The 'element' must be a dictionary previously
        validated so we know it has the mandatory 
        strcture.
        """
        return element.get(_Field.VOICE_NARRATION, None) is not None
    
    @staticmethod
    def validate(
        element: dict
    ):
        """
        Validate that the 'voice_narration' fields are
        set and valid for loading a voice narration from
        a file or for generating a new one.

        The 'element' must be a dictionary previously
        validated so we know it has the mandatory 
        structure.
        """
        voice_narration = element.get(_Field.VOICE_NARRATION.value, None)

        if voice_narration is not None:
            # Validate all fields are, at least, set
            ParameterValidator.validate_dict_has_keys('voice_narration', voice_narration, VoiceNarrationField.get_all_values())

            filename = voice_narration[VoiceNarrationField.FILENAME.value]
            text = voice_narration[VoiceNarrationField.TEXT.value]
            engine = voice_narration[VoiceNarrationField.ENGINE.value]
            language = voice_narration[VoiceNarrationField.LANGUAGE.value]
            narrator_name = voice_narration[VoiceNarrationField.NARRATOR_NAME.value]
            speed = voice_narration[VoiceNarrationField.SPEED.value]
            emotion = voice_narration[VoiceNarrationField.EMOTION.value]

            # Validate the combination of fields values are valid
            if (
                filename is not None and
                not VoiceNarrationFilenameField(filename).is_valid
            ):
                raise Exception('The provided "voice_narration" "filename" field is not a valid audio file.')

            if (
                filename is None and
                (
                    not VoiceNarrationTextField(text).is_valid or
                    not VoiceNarrationEngineField(engine).is_valid or
                    not VoiceNarrationLanguageField(language, engine).is_valid or
                    not VoiceNarrationNarratorNameField(narrator_name).is_valid or
                    not VoiceNarrationSpeedField(speed).is_valid or
                    not VoiceNarrationEmotionField(emotion).is_valid
                )
            ):
                raise Exception('At least one of the "voice_narration" parameters needed is not valid.')
            
    @staticmethod
    def get(
        voice_narration: dict,
        output_filename: Union[str, None] = None
    ) -> str:
        """
        Obtain the voice narration, store it locally 
        and return the filename string. The provided
        'voice_narration' parameter must be the dict
        within the component dict.
        """
        filename = voice_narration[VoiceNarrationField.FILENAME.value]

        # TODO: 'text' has to be sanitized before using
        if not voice_narration['text_sanitized']:
            parser = ShortcodeParser()
            parser.parse(voice_narration[VoiceNarrationField.TEXT.value])
            text = parser.text_sanitized_without_shortcodes
        else:
            text = voice_narration['text_sanitized']

        return (
            filename
            if filename is not None else
            VoiceNarrationConcept._generate(
                text = text,
                engine = voice_narration[VoiceNarrationField.ENGINE.value],
                narrator_name = voice_narration[VoiceNarrationField.NARRATOR_NAME.value],
                speed = voice_narration[VoiceNarrationField.SPEED.value],
                emotion = voice_narration[VoiceNarrationField.EMOTION.value],
                output_filename = output_filename
            )
        )

    @staticmethod
    def _generate(
        text: str,
        engine: VoiceNarrationEngine,
        narrator_name: VoiceNarrationNarratorName,
        speed: VoiceNarrationSpeed,
        emotion: VoiceNarrationEmotion,
        output_filename: Union[str, None] = None
    ) -> str:
        """
        Generate a voice narration with the given 'text'
        and parameters, and stores it locally with the
        given 'output_filename'.
        """
        engine = VoiceNarrationEngine.to_enum(engine)
        narrator_name = VoiceNarrationNarratorName.to_enum(narrator_name)
        speed = VoiceNarrationSpeed.to_enum(speed)
        emotion = VoiceNarrationEmotion.to_enum(emotion)

        # TODO: How to actually handle these 'speed' and
        # 'emotion' according to the 'engine' chosen (?)

        # TODO: Handle params, please
        return GoogleVoiceNarrator.narrate(
            text = text,
            output_filename = output_filename
        )
    
# TODO: Check yta-api because I'm validating it there


from yta_core.enums.field_v2 import _Field, MusicField
from yta_core.validation import MusicEngineField, MusicFilenameField, MusicKeywordsField, MusicUrlField
from yta_core.builder.music.enums import MusicEngine
from yta_core.builder.music.generator import YoutubeMusicGenerator
from yta_general_utils.file.checker import FileValidator
from yta_general_utils.downloader import Downloader
from yta_general_utils.programming.validator.parameter import ParameterValidator
from typing import Union


class MusicConcept:
    """
    Class to simplify the way we handle the 'music'
    for a component.

    This class must be used when the component json
    structure has been validated and it is ready for
    the building process.

    TODO: Check 'validation.py' for the validation
    process.
    """

    @staticmethod
    def do_should_be_handled(
        element: dict
    ) -> bool:
        """
        Check if the 'music' should be handled or not according
        to the parameters set in the given 'element'.
        """
        return element.get(_Field.MUSIC, None) is not None
    
    @staticmethod
    def validate(
        element: dict
    ):
        """
        Validate that the 'music' fields are set and
        valid for loading music from a file or for
        obtaining or creating the expected music.
        """
        music = element.get(_Field.MUSIC.value, None)

        if music is not None:
            # Validate all fields are, at least, set
            ParameterValidator.validate_dict_has_keys('music', music, MusicField.get_all_values())

            filename = music[MusicField.FILENAME.value]
            url = music[MusicField.URL.value]
            engine = music[MusicField.ENGINE.value]
            keywords = music[MusicField.KEYWORDS.value]

            # Validate the combination of fields values are valid
            if (
                filename is not None and
                not MusicFilenameField(filename).is_valid
            ):
                raise Exception('The provided "music" "filename" field is not a valid audio file.')

            if (
                filename is None and
                (
                    not MusicFilenameField(filename).is_valid or
                    not MusicUrlField(url).is_valid or
                    not MusicEngineField(engine).is_valid or
                    not MusicKeywordsField(keywords).is_valid
                )
            ):
                raise Exception('At least one of the "music" parameters needed is not valid.')
            
    @staticmethod
    def get(
        music: dict
    ) -> str:
        """
        Obtain the music, store it locally and return
        the filename string. The provided 'music'
        parameter must be the dict within the component
        dict.
        """
        filename = music[MusicField.FILENAME.value]
        url = music[MusicField.URL.value]
        keywords = music[MusicField.KEYWORDS.value]
        engine = music[MusicField.ENGINE.value]

        if filename is not None:
            music_filename = filename
            if not FileValidator.file_is_audio_file(music_filename):
                raise Exception('The provided "filename" is not an audio file.')
        elif url is not None:
            music_filename = Downloader.download_audio(url)
            # TODO: What about this? Should be done in advance
            # when the parameters are provided? It can be available
            # when provided but not when the video is actually being
            # generated...
            if not FileValidator.file_is_audio_file(music_filename):
                raise Exception('The provided "url" is not an audio file.')
        elif keywords is not None:
            music_filename = MusicConcept._generate(
                kewords = keywords,
                engine = MusicEngine.to_enum(engine)
            )

        return music_filename
            
    @staticmethod
    def _generate(
        keywords: str,
        engine: MusicEngine,
        output_filename: Union[str, None] = None
    ) -> Union[str, None]:
        ParameterValidator.validate_mandatory_string('keywords', keywords, do_accept_empty = False)
        engine = MusicEngine.to_enum(engine)

        # TODO: What if we make a method to turn 'DEFAULT'
        # option into another one (?)
        engine = (
            MusicEngine.YOUTUBE
            if engine == MusicEngine.DEFAULT else
            engine
        )

        return {
            MusicEngine.YOUTUBE: lambda keywords, output_filename: YoutubeMusicGenerator.generate(
                keywords = keywords,
                output_filename = output_filename
            )
        }[engine](keywords, output_filename)
        
    

    



"""
This module must be in another library I 
think, like the 'yta-audio' holds the audio
narration generation.
"""
from yta_core.builder.youtube import YoutubeDownloader
from abc import ABC, abstractmethod
from typing import Union


class MusicGenerator(ABC):
    """
    Class to wrap the generation of music for our
    videos.
    """

    @abstractmethod
    def generate(
        keywords: str,
        # TODO: Maybe more parameters
        output_filename: Union[str, None] = None
    ):
        pass

class YoutubeMusicGenerator(MusicGenerator):
    """
    Class that generates music by downloading it from
    our specific Youtube channel.
    """

    def generate(
        keywords: str,
        output_filename: Union[str, None] = None
    ) -> Union[str, None]:
        return YoutubeDownloader().download_music_audio(
            keywords = keywords,
            output_filename = output_filename
        )

# TODO: Here can be Suno AI or any other way
# to generate music




from yta_general_utils.programming.enum import YTAEnum as Enum


class _Field(Enum):
    """
    Enum class to represent the names of the
    fields we are able to handle in the app.

    This has been built to have all the names
    centralized in one place and to reuse in
    any app part in which it is needed.

    For internal use only. Use a more specific
    Enum class to interact with the different
    field names.
    """

    STATUS = 'status'
    """
    The status of an element, that will tell us
    if it is being constructed, waiting for it,
    etc.
    """
    SEGMENTS = 'segments'
    """
    The segments that a project includes and have
    to be built.
    """
    TYPE = 'type'
    """
    The type of some specific element. See the 
    specific type Enum classes.
    """
    TEXT = 'text'
    """
    The text that has to be written in some 
    segment with text content.
    """
    DURATION = 'duration'
    """
    The length of time that the element being
    referenced will be displayed or occurring.
    """
    KEYWORDS = 'keywords'
    """
    The terms or sentence to make a search on
    Youtube, to create the specific premade, etc.
    Keywords to identify the expected behaviour.
    """
    URL = 'url'
    """
    The url from which we can obtain a resource
    that has to be included as part (or the
    whole) content of the element.
    """
    FILENAME = 'filename'
    """
    The local filename of the file that has to
    be included as part (or the whole) content 
    of the element.
    """
    ENGINE = 'engine'
    """
    The engine that must be used to build the
    music or the voice narration.
    """
    LANGUAGE = 'language'
    """
    The language that must be used for the
    audio narration.
    """
    VOICE = 'voice'
    """
    The voice that has to be used to create the
    voice narration.
    """
    MUSIC = 'music'
    """
    The filename of a music that has to be included
    as an audio part for the content.
    """
    VOICE_NARRATION = 'voice_narration'
    """
    The voice narration object that includes the
    information of how to build the voice narration.
    """
    ENHANCEMENTS = 'enhancements'
    """
    The enhancements that have to be built for the
    content.
    """
    START = 'start'
    """
    The time moment in which the content has to start.
    """
    MODE = 'mode'
    """
    The mode in which the content has to be included.
    """
    EXTRA_PARAMS = 'extra_params'
    """
    The extra parameters we need to build some of the
    content.
    """
    # Building fields below
    TRANSCRIPTION = 'transcription'
    """
    The transcription of the audio narration if 
    existing.
    """
    SHORTCODES = 'shortcodes'
    """
    The shortcodes found in an audio narration.
    """
    AUDIO_FILENAME = 'audio_filename'
    """
    The filename of the audio that has been 
    generated by the system and has to be used
    as the audio content part.
    """
    AUDIO_CLIP = 'audio_clip'
    """
    The clip generated from the 'audio_filename'.
    """
    VIDEO_FILENAME = 'video_filename'
    """
    The filename of the video part that has 
    been generated by the system and has to be
    used as the video content part.
    """
    VIDEO_CLIP = 'video_clip'
    """
    The clip generated from the 'video_filename'.
    """
    FULL_FILENAME = 'full_filename'
    """
    The filename of the video and audio part put
    together, generated by the system, and that
    is the whole segment content with the effects
    and everything appliedo n it.
    """
    FULL_CLIP = 'full_clip'
    """
    The clip generated from the 'full_filename'.
    """
    FILENAME_PROCESSED = 'filename_processed'
    """
    The filename of a concept that has been 
    processed and built.
    """
    NARRATOR_NAME = 'narrator_name'
    """
    The name of the narrator voice that will be
    used for the provided voice narration engine.
    """
    SPEED = 'speed'
    """
    The speed that must be used when generating
    the voice narration with the given narrator
    name and engine.
    """
    EMOTION = 'emotion'
    """
    The emotion that must be used when generating
    the voice narration.
    """
    PITCH = 'pitch'
    """
    The pitch that must be used when generating
    the voice narration.
    """

class ProjectField(Enum):
    """
    Enum class to represent the names of the
    fields related to a Project.
    """

    STATUS = _Field.STATUS.value
    """
    The status of an element, that will tell us
    if it is being constructed, waiting for it,
    etc.

    See ProjectStatus Enum class.
    """
    SEGMENTS = _Field.SEGMENTS.value
    """
    The segments that a project includes and have
    to be built.
    """

class SegmentField(Enum):
    """
    Enum class to represent the names of the
    fields related to a Segment (which belongs
    to a Project).
    """

    TYPE = _Field.TYPE.value
    """
    The type of a segment, that determins what
    kind of content will be used to fulfill that
    segment.
    """
    TEXT = _Field.TEXT.value
    """
    The text that has to be written in some 
    segment with text content.
    """
    DURATION = _Field.DURATION.value
    """
    The length of time that the segment being
    referenced will be displayed or occurring.
    """
    KEYWORDS = _Field.KEYWORDS.value
    """
    The terms or sentence to make a search on
    Youtube, to create the specific premade, etc.
    Keywords to identify the expected behaviour.
    """
    URL = _Field.URL.value
    """
    The url from which we can obtain a resource
    that has to be included as part (or the
    whole) content of the element.
    """
    FILENAME = _Field.FILENAME.value
    """
    The local filename of the file that has to
    be included as part (or the whole) content 
    of the element.
    """
    MUSIC = _Field.MUSIC.value
    """
    The information (as a dict) of the music that
    must be used for this Segment (it can be None,
    which means no music requested).
    """
    VOICE_NARRATION = _Field.VOICE_NARRATION.value
    """
    The information (as a dict) of the voice
    narration that must be used for this Segment
    (it can be None, which means no voice narration
    requested).
    """
    ENHANCEMENTS = _Field.ENHANCEMENTS.value
    """
    The enhancements that have to be built for the
    content.
    """
    EXTRA_PARAMS = _Field.EXTRA_PARAMS.value
    """
    The extra parameters we need to build some of the
    content.
    """

class EnhancementField(Enum):
    """
    Enum class to represent the names of the
    fields related to an Enhancement (which
    belongs to a Project Segment).
    """

    TYPE = _Field.TYPE.value
    """
    The type of an Enhancement, that determins
    what kind of content will be used to fulfill
    that segment.
    """
    KEYWORDS = _Field.KEYWORDS.value
    """
    The terms or sentence to make a search on
    Youtube, to create the specific premade, etc.
    Keywords to identify the expected behaviour.
    """
    URL = _Field.URL.value
    """
    The url from which we can obtain a resource
    that has to be included as part (or the
    whole) content of the element.
    """
    FILENAME = _Field.FILENAME.value
    """
    The local filename of the file that has to
    be included as part (or the whole) content 
    of the element.
    """
    TEXT = _Field.TEXT.value
    """
    The text that has to be written in some 
    segment with text content.
    """
    DURATION = _Field.DURATION.value
    """
    The length of time that the segment being
    referenced will be displayed or occurring.
    """
    MUSIC = _Field.MUSIC.value
    """
    The information (as a dict) of the music that
    must be used for this Segment (it can be None,
    which means no music requested).
    """
    VOICE_NARRATION = _Field.VOICE_NARRATION.value
    """
    The information (as a dict) of the voice
    narration that must be used for this Segment
    (it can be None, which means no voice narration
    requested).
    """
    START = _Field.START.value
    """
    The time moment in which the content has to start.
    """
    MODE = _Field.MODE.value
    """
    The mode in which the content has to be included.
    """
    EXTRA_PARAMS = _Field.EXTRA_PARAMS.value
    """
    The extra parameters we need to build some of the
    content.
    """

class ShortcodeField(Enum):
    """
    Enum class to represent the names of the
    fields related to a Shortcode (which
    was detected within a Segment or Enhancement
    text).
    """

    TYPE = _Field.TYPE.value
    """
    The type of a Shortcode, that determins
    what kind of content will be used to fulfill
    that segment.
    """
    KEYWORDS = _Field.KEYWORDS.value
    """
    The terms or sentence to make a search on
    Youtube, to create the specific premade, etc.
    Keywords to identify the expected behaviour.
    """
    URL = _Field.URL.value
    """
    The url from which we can obtain a resource
    that has to be included as part (or the
    whole) content of the element.
    """
    FILENAME = _Field.FILENAME.value
    """
    The local filename of the file that has to
    be included as part (or the whole) content 
    of the element.
    """
    TEXT = _Field.TEXT.value
    """
    The text that has to be written in some 
    segment with text content.
    """
    DURATION = _Field.DURATION.value
    """
    The length of time that the segment being
    referenced will be displayed or occurring.
    """
    MUSIC = _Field.MUSIC.value
    """
    The information (as a dict) of the music that
    must be used for this Segment (it can be None,
    which means no music requested).
    """
    VOICE_NARRATION = _Field.VOICE_NARRATION.value
    """
    The information (as a dict) of the voice
    narration that must be used for this Segment
    (it can be None, which means no voice narration
    requested).
    """
    START = _Field.START.value
    """
    The time moment in which the content has to start.
    """
    MODE = _Field.MODE.value
    """
    The mode in which the content has to be included.
    """
    EXTRA_PARAMS = _Field.EXTRA_PARAMS.value
    """
    The extra parameters we need to build some of the
    content.
    """

class ProjectBuildingField(Enum):
    """
    The fields of a Project that are used when its
    content is being built and doesn't come from
    the original json source.
    """

    STATUS = _Field.STATUS.value
    """
    The status of the Project, that will tell us
    if it is being constructed, waiting for it,
    etc.
    """

class SegmentBuildingField(Enum):
    """
    The fields that are used when building the
    Segment and are not provided by the user in
    the initial json data.
    """

    STATUS = _Field.STATUS.value
    """
    The status of the Segment, that will tell us
    if it is being constructed, waiting for it,
    etc.
    """
    TRANSCRIPTION = _Field.TRANSCRIPTION.value
    """
    The transcription of the audio narration of
    the segment if existing.
    """
    SHORTCODES = _Field.SHORTCODES.value
    """
    The shortcodes that have been found in the
    audio narration.
    """
    AUDIO_FILENAME = _Field.AUDIO_FILENAME.value
    """
    The filename of the audio that has been 
    generated by the system and has to be used
    as the audio content part.
    """
    AUDIO_CLIP = _Field.AUDIO_CLIP.value
    """
    The clip generated from the 'audio_filename'.
    """
    VIDEO_FILENAME = _Field.VIDEO_FILENAME.value
    """
    The filename of the video part that has 
    been generated by the system and has to be
    used as the video content part.
    """
    VIDEO_CLIP = _Field.VIDEO_CLIP.value
    """
    The clip generated from the 'video_filename'.
    """
    FULL_FILENAME = _Field.FULL_FILENAME.value
    """
    The filename of the video and audio part put
    together, generated by the system, and that
    is the whole segment content with the effects
    and everything appliedo n it.
    """
    FULL_CLIP = _Field.FULL_CLIP.value
    """
    The clip generated from the 'full_filename'.
    """

class MusicField(Enum):
    """
    The fields that are used for the Segment or
    Enhancement music and are not provided by
    the user in the initial json data.
    """
    
    FILENAME = _Field.FILENAME.value
    """
    The filename of the music that must be used
    in the segment or enhancement.
    """
    URL = _Field.URL.value
    """
    The url in which the music is stored so it
    can be downloaded and applied in the segment
    or enhancement.
    """
    ENGINE = _Field.ENGINE.value
    """
    The engine that must be used to obtain the 
    music that has been requested for the segment
    or enhancement.
    """
    KEYWORDS = _Field.KEYWORDS.value
    """
    The keywords that must be used to obtain the
    music according to the engine provided.
    """

class MusicBuildingField(Enum):
    """
    The fields that are used for the Segment or
    Enhancement music and are not provided by
    the user in the initial json data.
    """

    FILENAME_PROCESSED = _Field.FILENAME_PROCESSED.value
    """
    The filename of the definitive music that
    has been processed and/or built and is able
    to be used for the Component.
    """

class VoiceNarrationField(Enum):
    """
    The fields that are used for the Segment or
    Enhancement voice narration and are not
    provided by the user in the initial json
    data.
    """

    FILENAME = _Field.FILENAME.value
    """
    The filename that holds a voice narration
    that must be used directly.
    """
    TEXT = _Field.TEXT.value
    """
    The text that must be used to be narrated
    by the voice narration.
    """
    ENGINE = _Field.ENGINE.value
    """
    The engine that must be used within the
    voice narration building process.
    """
    LANGUAGE = _Field.LANGUAGE.value
    """
    The language that must be used for the
    audio narration.
    """
    NARRATOR_NAME = _Field.NARRATOR_NAME.value
    """
    The voice of the narrator, for the given
    voice narration engine, that must be used.
    """
    SPEED = _Field.SPEED.value
    """
    The speed that must be used when generating
    the voice narration.
    """
    EMOTION = _Field.EMOTION.value
    """
    The emotion that must be used when
    generating the voice narration.
    """
    PITCH = _Field.PITCH.value
    """
    The pitch that must be used when generating
    the voice narration.
    """

class VoiceNarrationBuildingField(Enum):
    """
    The fields that are used for the Segment or
    Enhancement voice narration and are not
    provided by the user in the initial json data.
    """

    FILENAME_PROCESSED = _Field.FILENAME_PROCESSED.value
    """
    The filename of the definitive voice
    narration that has been processed and/or
    built and is able to be used for the Component.
    """

class EnhancementBuildingField(Enum):
    """
    The fields that are used when building the 
    Enhancement and are not provided by the user
    in the initial json data.
    """

    STATUS = _Field.STATUS.value
    """
    The status of the Segment, that will tell us
    if it is being constructed, waiting for it,
    etc.
    """
    TRANSCRIPTION = _Field.TRANSCRIPTION.value
    """
    The transcription of the audio narration of
    the segment if existing.
    """
    SHORTCODES = _Field.SHORTCODES.value
    """
    The shortcodes that have been found in the
    audio narration.
    """
    AUDIO_FILENAME = _Field.AUDIO_FILENAME.value
    """
    The filename of the audio that has been 
    generated by the system and has to be used
    as the audio content part.
    """
    AUDIO_CLIP = _Field.AUDIO_CLIP.value
    """
    The clip generated from the 'audio_filename'.
    """
    VIDEO_FILENAME = _Field.VIDEO_FILENAME.value
    """
    The filename of the video part that has 
    been generated by the system and has to be
    used as the video content part.
    """
    VIDEO_CLIP = _Field.VIDEO_CLIP.value
    """
    The clip generated from the 'video_filename'.
    """
    FULL_FILENAME = _Field.FULL_FILENAME.value
    """
    The filename of the video and audio part put
    together, generated by the system, and that
    is the whole segment content with the effects
    and everything appliedo n it.
    """
    FULL_CLIP = _Field.FULL_CLIP.value
    """
    The clip generated from the 'full_filename'.
    """


from yta_general_utils.programming.enum import YTAEnum as Enum


class _Field(Enum):
    """
    Enum class to represent the names of the
    fields we are able to handle in the app.

    This has been built to have all the names
    centralized in one place and to reuse in
    any app part in which it is needed.

    For internal use only. Use a more specific
    Enum class to interact with the different
    field names.
    """

    TYPE = 'type'
    """
    The type of some specific element. See the 
    specific type Enum classes.
    """
    TEXT = 'text'
    """
    The text that has to be written in some 
    segment with text content.
    """
    VOICE_NARRATION = 'voice_narration'
    """
    The dictionary that holds the information
    about the voice narration that will be
    used within the content we are creating.
    It can be loaded from a local file,
    created from zero, or not used.
    """
    MUSIC = 'music'
    """
    The dictionary that holds the information
    about the music that will be used within
    the content we are creating. It can be 
    loaded from a local file, created from
    scratch, downloaded from an external
    source or not used.
    """
    FILENAME = 'filename'
    """
    The local filename of the file that has to
    be included as part (or the whole) content 
    of the element.
    """
    URL = 'url'
    """
    The url from which we can obtain a resource
    that has to be included as part (or the
    whole) content of the element.
    """
    KEYWORDS = 'keywords'
    """
    The terms or sentence to make a search on
    Youtube, to create the specific premade, etc.
    Keywords to identify the expected behaviour.
    """
    TEXT = 'text'
    """
    The text that must be used and included in
    some kind of elements to be successfully
    generated.
    """
    DURATION = 'duration'
    """
    The length of time that the element being
    referenced will be displayed or occurring.
    """
    ENHANCEMENTS = 'enhancements'
    """
    The enhancements that have to be built for the
    content.
    """
    EXTRA_PARAMS = 'extra_params'
    """
    The extra parameters we need to build some of the
    content.
    """
    # Subfields (fields inside main fields)
    ENGINE = 'engine'
    """
    The engine that has to be used to generate some
    specific content.
    """
    LANGUAGE = 'language'
    """
    The language that has to be used to generate the
    voice narration.
    """
    NARRATOR_NAME = 'narrator_name'
    """
    The name of the voice that must narrate the text
    that must be narrated.
    """
    SPEED = 'speed'
    """
    The speed of the voice narration.
    """
    EMOTION = 'emotion'
    """
    The emotion we expect to be shown when narrating
    the text.
    """
    PITCH = 'pitch'
    """
    The pitch we expect to be shown when narrating
    the text.
    """
    # Other fields
    STATUS = 'status'
    """
    The status of an element, that will tell us
    if it is being constructed, waiting for it,
    etc.
    """
    SEGMENTS = 'segments'
    """
    The segments that a project includes and have
    to be built.
    """
    ENHANCEMENTS = 'enhancements'
    """
    The enhancements a Segment of a Project can
    have.
    """
    START = 'start'
    """
    The time moment in which the content has to start.
    """
    MODE = 'mode'
    """
    The mode in which the content has to be included.
    """
    # Building fields below
    TRANSCRIPTION = 'transcription'
    """
    The transcription of the audio narration if 
    existing.
    """
    SHORTCODES = 'shortcodes'
    """
    The shortcodes found in an audio narration.
    """
    AUDIO_FILENAME = 'audio_filename'
    """
    The filename of the audio that has been 
    generated by the system and has to be used
    as the audio content part.
    """
    AUDIO_CLIP = 'audio_clip'
    """
    The clip generated from the 'audio_filename'.
    """
    VIDEO_FILENAME = 'video_filename'
    """
    The filename of the video part that has 
    been generated by the system and has to be
    used as the video content part.
    """
    VIDEO_CLIP = 'video_clip'
    """
    The clip generated from the 'video_filename'.
    """
    FULL_FILENAME = 'full_filename'
    """
    The filename of the video and audio part put
    together, generated by the system, and that
    is the whole segment content with the effects
    and everything appliedo n it.
    """
    FULL_CLIP = 'full_clip'
    """
    The clip generated from the 'full_filename'.
    """

class ProjectField(Enum):
    """
    Enum class to represent the names of the
    fields related to a Project.
    """

    STATUS = _Field.STATUS.value
    """
    The status of an element, that will tell us
    if it is being constructed, waiting for it,
    etc.

    See ProjectStatus Enum class.
    """
    SEGMENTS = _Field.SEGMENTS.value
    """
    The segments that a project includes and have
    to be built.
    """

class SegmentField(Enum):
    """
    Enum class to represent the names of the
    fields related to a Segment (which belongs
    to a Project).
    """

    TYPE = _Field.TYPE.value
    """
    The type of a segment, that determins what
    kind of content will be used to fulfill that
    segment.
    """
    VOICE_NARRATION = _Field.VOICE_NARRATION.value
    MUSIC = _Field.MUSIC.value
    FILENAME = _Field.FILENAME.value
    URL = _Field.URL.value
    KEYWORDS = _Field.KEYWORDS.value
    TEXT = _Field.TEXT.value
    DURATION = _Field.DURATION.value
    ENHANCEMENTS = _Field.ENHANCEMENTS.value
    EXTRA_PARAMS = _Field.EXTRA_PARAMS.value

class EnhancementField(Enum):
    """
    Enum class to represent the names of the
    fields related to an Enhancement (which
    belongs to a Project Segment).
    """

    TYPE = _Field.TYPE.value
    VOICE_NARRATION = _Field.VOICE_NARRATION.value
    MUSIC = _Field.MUSIC.value
    FILENAME = _Field.FILENAME.value
    URL = _Field.URL.value
    KEYWORDS = _Field.KEYWORDS.value
    TEXT = _Field.TEXT.value
    DURATION = _Field.DURATION.value
    START = _Field.START.value
    """
    The time moment in which the content has to start.
    """
    MODE = _Field.MODE.value
    """
    The mode in which the content has to be included.
    """
    EXTRA_PARAMS = _Field.EXTRA_PARAMS.value

class ShortcodeField(Enum):
    """
    Enum class to represent the names of the
    fields related to a Shortcode (which
    was detected within a Segment or Enhancement
    text).
    """

    TYPE = _Field.TYPE.value
    VOICE_NARRATION = _Field.VOICE_NARRATION.value
    MUSIC = _Field.MUSIC.value
    FILENAME = _Field.FILENAME.value
    URL = _Field.URL.value
    KEYWORDS = _Field.KEYWORDS.value
    TEXT = _Field.TEXT.value
    DURATION = _Field.DURATION.value
    START = _Field.START.value
    """
    The time moment in which the content has to start.
    """
    MODE = _Field.MODE.value
    """
    The mode in which the content has to be included.
    """
    EXTRA_PARAMS = _Field.EXTRA_PARAMS.value

class ProjectBuildingField(Enum):
    """
    The fields of a Project that are used when its
    content is being built and doesn't come from
    the original json source.
    """

    STATUS = _Field.STATUS.value
    """
    The status of the Project, that will tell us
    if it is being constructed, waiting for it,
    etc.
    """

class SegmentBuildingField(Enum):
    """
    The fields that are used when building the
    Segment and are not provided by the user in
    the initial json data.
    """

    STATUS = _Field.STATUS.value
    """
    The status of the Segment, that will tell us
    if it is being constructed, waiting for it,
    etc.
    """
    TRANSCRIPTION = _Field.TRANSCRIPTION.value
    """
    The transcription of the audio narration of
    the segment if existing.
    """
    SHORTCODES = _Field.SHORTCODES.value
    """
    The shortcodes that have been found in the
    audio narration.
    """
    AUDIO_FILENAME = _Field.AUDIO_FILENAME.value
    """
    The filename of the audio that has been 
    generated by the system and has to be used
    as the audio content part.
    """
    AUDIO_CLIP = _Field.AUDIO_CLIP.value
    """
    The clip generated from the 'audio_filename'.
    """
    VIDEO_FILENAME = _Field.VIDEO_FILENAME.value
    """
    The filename of the video part that has 
    been generated by the system and has to be
    used as the video content part.
    """
    VIDEO_CLIP = _Field.VIDEO_CLIP.value
    """
    The clip generated from the 'video_filename'.
    """
    FULL_FILENAME = _Field.FULL_FILENAME.value
    """
    The filename of the video and audio part put
    together, generated by the system, and that
    is the whole segment content with the effects
    and everything appliedo n it.
    """
    FULL_CLIP = _Field.FULL_CLIP.value
    """
    The clip generated from the 'full_filename'.
    """

class EnhancementBuildingField(Enum):
    """
    The fields that are used when building the 
    Enhancement and are not provided by the user
    in the initial json data.
    """

    STATUS = _Field.STATUS.value
    """
    The status of the Segment, that will tell us
    if it is being constructed, waiting for it,
    etc.
    """
    TRANSCRIPTION = _Field.TRANSCRIPTION.value
    """
    The transcription of the audio narration of
    the segment if existing.
    """
    SHORTCODES = _Field.SHORTCODES.value
    """
    The shortcodes that have been found in the
    audio narration.
    """
    AUDIO_FILENAME = _Field.AUDIO_FILENAME.value
    """
    The filename of the audio that has been 
    generated by the system and has to be used
    as the audio content part.
    """
    AUDIO_CLIP = _Field.AUDIO_CLIP.value
    """
    The clip generated from the 'audio_filename'.
    """
    VIDEO_FILENAME = _Field.VIDEO_FILENAME.value
    """
    The filename of the video part that has 
    been generated by the system and has to be
    used as the video content part.
    """
    VIDEO_CLIP = _Field.VIDEO_CLIP.value
    """
    The clip generated from the 'video_filename'.
    """
    FULL_FILENAME = _Field.FULL_FILENAME.value
    """
    The filename of the video and audio part put
    together, generated by the system, and that
    is the whole segment content with the effects
    and everything appliedo n it.
    """
    FULL_CLIP = _Field.FULL_CLIP.value
    """
    The clip generated from the 'full_filename'.
    """

class VoiceNarrationField(Enum):
    """
    The fields that are included within the
    'voice_narration' dict.
    """
    FILENAME = _Field.FILENAME.value
    TEXT = _Field.TEXT.value
    ENGINE = _Field.ENGINE.value
    LANGUAGE = _Field.LANGUAGE.value
    NARRATOR_NAME = _Field.NARRATOR_NAME.value
    SPEED = _Field.SPEED.value
    EMOTION = _Field.EMOTION.value
    PITCH = _Field.PITCH.value

class MusicField(Enum):
    """
    The fields that are included within the
    'music' dict.
    """
    FILENAME = _Field.FILENAME.value
    URL = _Field.URL.value
    ENGINE = _Field.ENGINE.value
    KEYWORDS = _Field.KEYWORDS.value


"""
This is a complex parameter which includes
different attributes to make possible the
voice narration construction.

This parameter can be None if no voice
narration is requested, or can have different
values according to the expected way of
building it.
"""
from yta_audio.voice.enums import VoiceEmotion, VoiceNarrationEngine, VoicePitch, VoiceSpeed, NarrationLanguage
from dataclasses import dataclass
from typing import Union

import json


@dataclass
class VoiceNarrationJson:
    """
    @dataclass
    The information the Music parameter must hold.
    This information is about a music that is going
    to be added in a video.
    """

    filename: str
    text: str
    engine: Union[VoiceNarrationEngine, None]
    language: Union[NarrationLanguage, None]
    # Narrator names are specific for each engine,
    # we don't have a common Enum
    narrator_name: Union[str, None]
    speed: Union[VoiceSpeed, None]
    emotion: Union[VoiceEmotion, None]
    pitch: Union[VoicePitch, None]
    text_sanitized: str
    """
    The text but sanitized, which means that any
    unexpected mark, accent, double space or 
    whatever makes the narration malfunction has
    been removed.
    """
    text_sanitized_without_shortcodes: str
    text_with_simplified_shortcodes: str

    def __init__(self):
        """
        Initializes the instance with empty string
        values for all the attributes so we can
        fill them later and individually when
        processing.
        """
        self.filename = ''
        self.text = ''
        self.engine = None
        self.language = None
        self.narrator_name = None
        self.speed = None
        self.emotion = None
        self.pitch = None
        # TODO: I need the sanitized without shortcodes,
        # but what about the other ones? are needed here (?)
        self.text_sanitized = ''
        self.text_sanitized_without_shortcodes = ''
        self.text_with_simplified_shortcodes = ''

    @property
    def as_dict(self):
        """
        Get the instance as a dictionary.
        """
        return {
            'filename': self.filename,
            'text': self.text,
            'engine': (
                self.engine.value
                if self.engine is not None else
                None
            ),
            'language': {
                self.language.value
                if self.language is not None else
                None
            },
            # 'narrator_name': (
            #     self.narrator_name.value
            #     if self.narrator_name is not None else
            #     None
            # ),
            # Narrator name is different for each engine
            # so we don't have a common Enum
            'narrator_name': self.narrator_name,
            'speed': (
                self.speed.value
                if self.speed is not None else
                None
            ),
            'emotion': (
                self.emotion.value
                if self.emotion is not None else
                None
            ),
            'pitch': {
                self.pitch.value
                if self.pitch is not None else
                None
            },
            'text_sanitized': self.text_sanitized,
            'text_sanitized_without_shortcodes': self.text_sanitized_without_shortcodes,
            'text_with_simplified_shortcodes': self.text_with_simplified_shortcodes
        }

    @property
    def as_json(self):
        """
        Get the instance as a json, transformed
        from the dictionary version.
        """
        return json.dumps(self.as_dict)